/*
**> ex2.prg
**
** Gourlay, A.R. and S. McKee [1977], The construction of hopscotch methods
** for parabolic and elliptic equations in two space dimensions with a mixed
** derivative, Journal of Computational and Applied Mathematics, 3, 201-205
**
** page 203, example 2
**
** a(t,x,y) = 0.5*x^2 + y^2
** b(t,x,y) = -0.5 * (x^2 + y^2)
** c(t,x,y) = x^2 + 0.5*y^2
** d(t,x,y) = 0
** e(t,x,y) = 0
** f(t,x,y) = 0
** g(t,x,y) = 0
**
** u(0,x,y) = x^2*y + x*y^2
** u(t,0,y) = 0
** u(t,1,y) = (y + y^2) * exp(-t)
** u(t,x,0) = 0
** u(t,x,1) = (x + x^2) * exp(-t)
**
** h(t,x,y,u) = u
*/

new;
library PDE2D,pgraph;
PDE2Dset;


proc aProc(t,x,y);
  retp(0.5*x^2 + y^2);
endp;

proc bProc(t,x,y);
  retp( -0.5 * (x^2 + y^2) );
endp;

proc cProc(t,x,y);
  retp(x^2 + 0.5*y^2);
endp;

proc dProc(t,x,y);
  retp( zeros(rows(x),cols(y)) );
endp;

proc eProc(t,x,y);
  retp( zeros(rows(x),cols(y)) );
endp;

proc fProc(t,x,y);
  retp( zeros(rows(x),cols(y)) );
endp;

proc gProc(t,x,y);
  retp( zeros(rows(x),cols(y)) );
endp;

proc tminBound(t,x,y);
  retp( (x^2).*y + x.*(y^2) );
endp;

proc xminBound(t,x,y);
  retp( zeros(rows(x),cols(y)) );
endp;

proc xmaxBound(t,x,y);
  retp( (y + y^2) * exp(-t) );
endp;

proc yminBound(t,x,y);
  retp( zeros(rows(x),cols(y)) );
endp;

proc ymaxBound(t,x,y);
  retp( (x + x^2) * exp(-t) );
endp;


call PDE2D(&aProc,&bProc,&cProc,&dProc,&eProc,&fProc,&gProc,0,
               &tminBound,&xminBound,&xmaxBound,&yminBound,&ymaxBound,
               0,0,0,0);

xmin = 0; xmax = 1; Nx = 21;
ymin = 0; ymax = 1; Ny = 21;
tmin = 0; tmax = 0.25; Nt = 51;

call Hopscotch(tmin,tmax,Nt,xmin,xmax,Nx,ymin,ymax,Ny,"ex2");

x = ExtractSolution("ex2","x");
y = ExtractSolution("ex2","y");
t = ExtractSolution("ex2","t");
U = ExtractSolution("ex2","t"|tmax);

/* The exact solution is 
**
**          u(t,x,y) = (x^2*y + x*y^2) * exp(-t)
**
*/

proc solutionProc(t,x,y);
  retp( ( (x^2).*y + x.*(y^2) ) * exp(-t) );
endp;

sol = solutionProc(tmax,x,y);

graphset;

  begwind;
  makewind(9,1,0,6.855-1,1);
  makewind(9/2,5.855,0,0,1);
  makewind(9/2,5.855,9/2,0,1);

  setwind(1);
    _pdate = ""; _ptitlht = 1.25; _pnum = 0; _paxes = 0;
    title("Gourlay and McKee @[1977@] - Example 2 page 203");
    draw;

  graphset;
    _pdate = ""; _pnum = 2; _ptitlht = 0.25; _paxht = 0.20; _pnumht = 0.15;
    xlabel("x");
    ylabel("y");

  setwind(2);
    title("Numerical solution")
    surface(x',y',U');

  setwind(3);
    title("Exact solution")
    surface(x',y',sol');
    graphprt("-c=1 -cf=ex2.eps");

  endwind;
