/*
**> ex7.prg
**
** Laplace Equation
*/

new;
library PDE2D,pgraph;
PDE2Dset;

proc aProc(t,x,y);
  retp( 2*ones(rows(x),cols(y)) );
endp;

proc bProc(t,x,y);
  retp( zeros(rows(x),cols(y)) );
endp;

proc cProc(t,x,y);
  retp( 2*ones(rows(x),cols(y)) );
endp;

proc dProc(t,x,y);
  retp( zeros(rows(x),cols(y)) );
endp;

proc eProc(t,x,y);
  retp( zeros(rows(x),cols(y)) );
endp;

proc fProc(t,x,y);
  retp( zeros(rows(x),cols(y)) );
endp;

proc gProc(t,x,y);
  retp( zeros(rows(x),cols(y)) );
endp;

proc InitialGuessSolution(t,x,y);
  retp( rndu(rows(x),cols(y)) );
endp;

proc xminBound(t,x,y);
  retp( ones(rows(x),cols(y)) .* (1 - y^2) );
endp;

proc xmaxBound(t,x,y);
  retp( ones(rows(x),cols(y)) .* (1 - y^2) );
endp;

proc yminBound(t,x,y);
  retp( ones(rows(x),cols(y)) .* (x^2 - 1) );
endp;

proc ymaxBound(t,x,y);
  retp( ones(rows(x),cols(y)) .* (x^2 - 1) );
endp;

call PDE2D(&aProc,&bProc,&cProc,&dProc,&eProc,&fProc,&gProc,0,
               &InitialGuessSolution,&xminBound,&xmaxBound,&yminBound,&ymaxBound,
               0,0,0,0);

xmin = -1; xmax = 1; Nx = 21;
ymin = -1; ymax = 1; Ny = 21;
tmin = 0; tmax = 0.25; Nt = 101;

_Hopscotch_PrintIters = 2;
_Hopscotch_Elliptic = 1;
_Hopscotch_SaveLastIter = 1;
_fcmptol = 1e-5;

_Hopscotch_Methods[1] = 2;  /* Center Filling Method */

_Hopscotch_Methods[2] = 0.5; /* Crank-Nicholson Theta Method */

call Hopscotch(tmin,tmax,Nt,xmin,xmax,Nx,ymin,ymax,Ny,"tmp");
x = ExtractSolution("tmp","x");
y = ExtractSolution("tmp","y");
t = ExtractSolution("tmp","t");
U1 = ExtractSolution("tmp","t"|t);

_Hopscotch_Methods[2] = 1; /* Implicit Theta Method */

call Hopscotch(tmin,tmax,Nt,xmin,xmax,Nx,ymin,ymax,Ny,"tmp");
t = ExtractSolution("tmp","t");
U2 = ExtractSolution("tmp","t"|t);

_Hopscotch_Methods[2] = 2; /* Ordered Odd-Even Theta Method */

call Hopscotch(tmin,tmax,Nt,xmin,xmax,Nx,ymin,ymax,Ny,"tmp");
t = ExtractSolution("tmp","t");
U3 = ExtractSolution("tmp","t"|t);

_Hopscotch_Methods[2] = 3; /* Line Theta Method */

call Hopscotch(tmin,tmax,Nt,xmin,xmax,Nx,ymin,ymax,Ny,"tmp");
t = ExtractSolution("tmp","t");
U4 = ExtractSolution("tmp","t"|t);

proc SolutionProc(x,y);
  retp(x^2-y^2);
endp;

sol = SolutionProc(x,y);


graphset;
  begwind;
  makewind(9,6.855,0,0,1);
  makewind(2.5,2.5,0,0,1);
  makewind(2.5,2.5,0,6.855-2.5,1);
  makewind(2.5,2.5,9-2.5,0,1);
  makewind(2.5,2.5,9-2.5,6.855-2.5,1);

  _pnum = 2;

  setwind(1);     

    title("Laplace Equation");
    xtics(-1,1,0.5,0);
    ytics(-1,1,0.5,0);
    surface(x',y',U1');

  graphset;
  _ptitlht = 0.30; _pnumht = 0.70; _paxes = 0;

  setwind(2);
   title("Crank-Nicholson"); 
    contour(x',y',(U1-sol)');

  setwind(3);
    title("Implicit");
    contour(x',y',(U2-sol)');

  setwind(4);
    title("Ordered Odd-Even");
    contour(x',y',(U3-sol)');

  setwind(5);
    title("Line");
    contour(x',y',(U4-sol)');

  graphprt("-c=1 -cf=tmp.eps");

  endwind;
